"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = (0, tslib_1.__importDefault)(require("chalk"));
const promises_1 = require("node:fs/promises");
const node_fs_1 = require("node:fs");
const path_1 = require("path");
const definition_1 = require("../definition");
const prompts_1 = require("../core/utils/prompts");
const command_1 = (0, tslib_1.__importDefault)(require("../command"));
const flagsBuilder = (0, tslib_1.__importStar)(require("../flags"));
const args_1 = require("../args");
const cli_1 = require("../cli");
class Overlay extends command_1.default {
    async run() {
        const { args, flags } = this.parse(Overlay);
        const outputPath = flags.out;
        cli_1.cli.action.start("* Let's apply the overlay to the main definition");
        const api = await definition_1.API.load(args.FILE);
        await api.applyOverlay(args.OVERLAY_FILE);
        const [overlayedDefinition] = api.extractDefinition(outputPath);
        cli_1.cli.action.stop();
        if (outputPath) {
            await (0, promises_1.mkdir)((0, path_1.dirname)(outputPath), { recursive: true });
            let confirm = true;
            if ((0, node_fs_1.existsSync)(outputPath)) {
                await (0, prompts_1.confirm)(`Do you want to override the existing destination file? (${outputPath})`).catch(() => {
                    confirm = false;
                });
            }
            if (confirm) {
                await (0, promises_1.writeFile)(outputPath, overlayedDefinition);
            }
        }
        else {
            cli_1.cli.log(overlayedDefinition);
        }
        return;
    }
}
exports.default = Overlay;
Overlay.description = 'Apply an OpenAPI specified overlay to your API definition.';
Overlay.examples = [
    `Apply the OVERLAY_FILE to the existing DEFINITION_FILE. The resulting
definition is output on stdout meaning you can redirect it to a new
file.

${chalk_1.default.dim('$ bump overlay DEFINITION_FILE OVERLAY_FILE > destination/file.json')}
* Let's apply the overlay to the main definition... done
`,
];
Overlay.flags = {
    help: flagsBuilder.help({ char: 'h' }),
    out: flagsBuilder.out(),
};
Overlay.args = [args_1.fileArg, args_1.overlayFileArg];
